import assert from 'assert';
import { readFile, readdir, stat, writeFile } from 'fs-extra';
import path from 'path';
import { kJSExtractImportFromRegex, kJSMultilineCommentEnd, kJSSingleLineCommentStart, kNewlineChar, kTSFilepathRegex, } from './constants.js';
import { kProcessedFileOutcome, } from './types.js';
export function isImportPart(part) {
    return kJSExtractImportFromRegex.test(part);
}
export function isCommentEnd(part) {
    return (part.startsWith(kJSSingleLineCommentStart) ||
        part.endsWith(kJSMultilineCommentEnd));
}
export async function startProcessedFile(filepath) {
    const raw = await readFile(filepath, 'utf-8');
    const parts = raw.split(kNewlineChar);
    return {
        filepath,
        outcome: kProcessedFileOutcome.processed,
        originalParts: parts,
        workingParts: parts,
        isModified: false,
    };
}
export function startErrorProcessedFile(filepath, error) {
    return {
        error,
        filepath,
        outcome: kProcessedFileOutcome.error,
    };
}
export function startSkippedProcessedFile(filepath, skipReason) {
    return {
        filepath,
        skipReason,
        outcome: kProcessedFileOutcome.skip,
    };
}
export function setProcessedFileWorkingParts(f, parts, isModified) {
    assert(f.outcome === kProcessedFileOutcome.processed);
    f.workingParts = parts;
    f.isModified = isModified;
}
export function setProcessedFileExplicitModifications(f, mods) {
    if (!mods.length) {
        return;
    }
    assert(f.outcome === kProcessedFileOutcome.processed);
    f.explicitModifications = mods;
    f.workingParts = mods.map(mod => mod.modifiedPart);
    f.isModified = true;
}
export async function writeProcessedFile(f) {
    assert(f.outcome === kProcessedFileOutcome.processed);
    if (!f.isModified) {
        return;
    }
    const raw = f.workingParts.join(kNewlineChar);
    await writeFile(f.filepath, raw);
}
export function isTSFilepath(filepath) {
    return kTSFilepathRegex.test(filepath);
}
export async function traverseAndProcessTSFilesInTree(folderpath, handleFile) {
    // TODO: we may want to output the processed files are we traverse for
    // memory's sake
    let fList = [];
    const dirList = await readdir(folderpath, { withFileTypes: true });
    await Promise.all(dirList.map(async (entry) => {
        const entryPath = path.normalize(path.join(entry.path, entry.name));
        let f;
        try {
            if (entry.isDirectory()) {
                const result = await traverseAndProcessTSFilesInTree(entryPath, handleFile);
                fList = fList.concat(result);
            }
            else if (entry.isFile()) {
                if (isTSFilepath(entryPath)) {
                    f = await handleFile(entry, entryPath);
                }
            }
        }
        catch (error) {
            f = startErrorProcessedFile(entryPath, error);
        }
        finally {
            if (f) {
                fList.push(f);
            }
        }
    }));
    return fList;
}
export async function outputProcessedFiles(fList) {
    await Promise.all(fList.map(writeProcessedFile));
}
export async function logProcessedFiles(fList) {
    fList.forEach(f => {
        var _a;
        switch (f.outcome) {
            case kProcessedFileOutcome.skip:
                console.log('skipped ' + f.filepath);
                console.log('\t' + f.skipReason);
                break;
            case kProcessedFileOutcome.error:
                console.log('error ' + f.filepath);
                console.error('\t' + f.error);
                break;
            case kProcessedFileOutcome.processed:
                console.log('skipped ' + f.filepath);
                if ((_a = f.explicitModifications) === null || _a === void 0 ? void 0 : _a.length) {
                    f.explicitModifications.forEach(mod => {
                        // TODO: partIndex may not be correct anymore, because further
                        // processing could have been done past setting
                        // explicitModifications
                        console.log(`\tL${mod.partIndex} ${mod.prev} >> ${mod.after}`);
                    });
                }
        }
    });
}
export async function isFile(filepath) {
    try {
        const info = await stat(filepath);
        return info.isFile();
    }
    catch (error) {
        // console.error(error);
        return false;
    }
}
//# sourceMappingURL=utils.js.map